@@ -1,6 +1,9 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2009-2012 Oak Ridge National Laboratory.  All rights reserved.
  * Copyright (c) 2009-2012 Mellanox Technologies.  All rights reserved.
+ * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,13 +23,9 @@ int mca_bcol_base_init(bool enable_progress_threads, bool enable_mpi_threads)
 {
     mca_bcol_base_component_t *bcol_component;
     mca_base_component_list_item_t *cli;
-    opal_list_item_t *item;
     int ret;
 
-    for (item = opal_list_get_first((opal_list_t *) &mca_bcol_base_components_in_use);
-            opal_list_get_end((opal_list_t *) &mca_bcol_base_components_in_use) != item;
-            item = opal_list_get_next(item)) {
-        cli = (mca_base_component_list_item_t *) item;
+    OPAL_LIST_FOREACH(cli, &mca_bcol_base_components_in_use, mca_base_component_list_item_t) {
         bcol_component = (mca_bcol_base_component_t *) cli->cli_component;
 
         if (false == bcol_component->init_done) {
