@@ -1,5 +1,5 @@
 /**
-  Copyright (c) 2010 Voltaire, Inc. All rights reserved.
+  Copyright (c) 2011 Mellanox Technologies. All rights reserved.
   $COPYRIGHT$
 
   Additional copyrights may follow
@@ -10,6 +10,7 @@
 #include "ompi_config.h"
 #include "ompi/constants.h"
 #include "coll_fca.h"
+#include "coll_fca_convertor.h"
 
 
 static mca_coll_fca_dtype_info_t* mca_coll_fca_get_dtype(ompi_datatype_t *dtype)
@@ -19,24 +20,31 @@ static mca_coll_fca_dtype_info_t* mca_coll_fca_get_dtype(ompi_datatype_t *dtype)
     int id = dtype->id;
     int fca_dtype;
 
-    if (id < 0 || id >= FCA_DT_MAX_PREDEFINED)
+    if (id < 0 || id >= FCA_DT_MAX_PREDEFINED) {
         return NULL;
+    }
 
+    /* Different dtype structures may have the same id. In that case, we assume
+     * they are aliases.
+     */
     dtype_info = &mca_coll_fca_component.fca_dtypes[id];
-    if (dtype_info->mpi_dtype == dtype)
+    if (dtype_info->mpi_dtype->id == id) {
         return dtype_info;
+    }
 
     /* assert we don't overwrite another datatype */
     assert(dtype_info->mpi_dtype == MPI_DATATYPE_NULL);
-    fca_dtype = mca_coll_fca_component.fca_ops.translate_mpi_dtype(dtype->name);
-    if (fca_dtype < 0)
+
+    fca_dtype = fca_translate_mpi_dtype(dtype->name);
+    if (fca_dtype < 0) {
         return NULL;
+    }
 
     FCA_DT_GET_TRUE_EXTENT(dtype, &lb, &extent);
     dtype_info->mpi_dtype = dtype;
     dtype_info->mpi_dtype_extent = extent;
     dtype_info->fca_dtype = fca_dtype;
-    dtype_info->fca_dtype_extent = mca_coll_fca_component.fca_ops.get_dtype_size(fca_dtype);
+    dtype_info->fca_dtype_extent = fca_get_dtype_size(fca_dtype);
     FCA_VERBOSE(2, "Added new dtype[%d]: %s fca id: %d, mpi size: %lu, fca size: %lu",
                 id, dtype->name, dtype_info->fca_dtype, dtype_info->mpi_dtype_extent,
                 dtype_info->fca_dtype_extent);
@@ -58,7 +66,7 @@ static mca_coll_fca_op_info_t *mca_coll_fca_get_op(ompi_op_t *op)
         if (op_info->mpi_op == op) {
             return op_info;
         } else if (op_info->mpi_op == MPI_OP_NULL) {
-            fca_op = mca_coll_fca_component.fca_ops.translate_mpi_op(op->o_name);
+            fca_op = fca_translate_mpi_op(op->o_name);
             if (fca_op < 0)
                 return NULL;
             op_info->mpi_op = op;
@@ -72,19 +80,24 @@ static mca_coll_fca_op_info_t *mca_coll_fca_get_op(ompi_op_t *op)
     return NULL;
 }
 
-static int mca_coll_fca_get_buf_size(ompi_datatype_t *dtype, int count,
-                                     int contiguous_count)
+/**
+ * If "datatype" is contiguous when it appears "count" times, return 1 and
+ * set "*size" to the total buffer size, and *gap to the gap before the data.
+ * Otherwise return 0.
+ */
+static inline int mca_coll_fca_array_size(ompi_datatype_t *dtype, int count,
+                                          size_t *gap, size_t *size)
 {
     ptrdiff_t true_lb, true_extent;
 
-   /* Check that the type in contiguous */
-   if (!FCA_DT_IS_CONTIGUOUS_MEMORY_LAYOUT(dtype, contiguous_count)) {
-       FCA_VERBOSE(5, "Unsupported datatype layout, only contiguous is supported now");
-       return OMPI_ERROR;
-   }
-
-   FCA_DT_GET_TRUE_EXTENT(dtype, &true_lb, &true_extent);
-   return true_extent * count;
+    if (FCA_DT_IS_CONTIGUOUS_MEMORY_LAYOUT(dtype, count)) {
+        FCA_DT_GET_TRUE_EXTENT(dtype, &true_lb, &true_extent);
+        *gap = true_lb;
+        *size = true_extent * count;
+        return 1;
+    } else {
+        return 0;
+    }
 }
 
 static int mca_coll_fca_fill_reduce_spec(int count, ompi_datatype_t *dtype,
@@ -140,12 +153,16 @@ int mca_coll_fca_barrier(struct ompi_communicator_t *comm,
     int ret;
 
     FCA_VERBOSE(5,"Using FCA Barrier");
-    ret = mca_coll_fca_component.fca_ops.do_barrier(fca_module->fca_comm);
+    if (OPAL_UNLIKELY(ompi_mpi_finalize_started)) {
+        FCA_VERBOSE(5, "In finalize, reverting to previous barrier");
+        goto orig_barrier;
+    }
+    ret = fca_do_barrier(fca_module->fca_comm);
     if (ret < 0) {
         if (ret == -EUSEMPI) {
             goto orig_barrier;
         }
-        FCA_ERROR("Barrier failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Barrier failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
     return OMPI_SUCCESS;
@@ -165,31 +182,63 @@ int mca_coll_fca_bcast(void *buff, int count, struct ompi_datatype_t *datatype,
                        mca_coll_base_module_t *module)
 {
     mca_coll_fca_module_t *fca_module = (mca_coll_fca_module_t*)module;
+    MCA_COLL_FCA_DECLARE_CONVERTOR(conv);
     fca_bcast_spec_t spec;
+    size_t gap, size;
     int ret;
 
-    FCA_VERBOSE(5,"[%d] Calling mca_coll_fca_bcast, root=%d, count=%d",
+    FCA_VERBOSE(5, "[%d] Calling mca_coll_fca_bcast, root=%d, count=%d",
                 ompi_comm_rank(comm), root, count);
 
-    spec.size = mca_coll_fca_get_buf_size(datatype, count, count);
-    if (spec.size < 0 || spec.size > fca_module->fca_comm_caps.max_payload) {
-        FCA_VERBOSE(5, "Unsupported bcast operation, dtype=%s[%d] using fallback\n",
-                    datatype->name, count);
-        goto orig_bcast;
+    /* Setup exchange buffer */
+    spec.root = root;
+    if (mca_coll_fca_array_size(datatype, count, &gap, &size)) {
+        spec.buf = (char*)buff + gap;
+    } else {
+        mca_coll_fca_convertor_create(&conv, datatype, count, buff,
+                                      (root == fca_module->rank)
+                                                    ? MCA_COLL_FCA_CONV_SEND
+                                                    : MCA_COLL_FCA_CONV_RECV,
+                                      &spec.buf, &size);
     }
 
-    FCA_VERBOSE(5,"Using FCA Bcast");
-    spec.buf  = buff;
-    mca_coll_fca_get_bcast_root(root, fca_module->local_ranks,
-                                fca_module->num_local_procs, &spec);
-    ret = mca_coll_fca_component.fca_ops.do_bcast(fca_module->fca_comm, &spec);
+    /* Check that operation size does not exceed limit */
+    spec.size = size;
+    if (spec.size > fca_module->fca_comm_caps.max_payload) {
+         FCA_VERBOSE(5, "Unsupported bcast operation size %d, using fallback",
+                     spec.size);
+         if (spec.buf != buff) {
+             mca_coll_fca_convertor_destroy(&conv);
+         }
+         goto orig_bcast;
+    }
+
+    /* Sender may pack data */
+    if (spec.buf != buff && root == fca_module->rank) {
+        mca_coll_fca_convertor_process(&conv, 0);
+    }
+
+    /* Call FCA Bcast */
+    FCA_VERBOSE(5, "Using FCA Bcast");
+    ret = fca_do_bcast(fca_module->fca_comm, &spec);
+
+    /* Destroy convertor if operation failed */
     if (ret < 0) {
+        mca_coll_fca_convertor_destroy(&conv);
         if (ret == -EUSEMPI) {
             goto orig_bcast;
         }
-        FCA_ERROR("Bcast failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Bcast failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
+
+    /* Unpack data and clean up convertor */
+    if (mca_coll_fca_convertor_valid(&conv)) {
+        if (root != fca_module->rank) {
+            mca_coll_fca_convertor_process(&conv, 0);
+        }
+        mca_coll_fca_convertor_destroy(&conv);
+    }
     return OMPI_SUCCESS;
 
 orig_bcast:
@@ -224,12 +273,12 @@ int mca_coll_fca_reduce(void *sbuf, void *rbuf, int count,
     }
 
     FCA_VERBOSE(5,"Using FCA Reduce");
-    ret = mca_coll_fca_component.fca_ops.do_reduce(fca_module->fca_comm, &spec);
+    ret = fca_do_reduce(fca_module->fca_comm, &spec);
     if (ret < 0) {
         if (ret == -EUSEMPI) {
             goto orig_reduce;
         }
-        FCA_ERROR("Reduce failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Reduce failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
     return OMPI_SUCCESS;
@@ -265,19 +314,60 @@ int mca_coll_fca_allreduce(void *sbuf, void *rbuf, int count,
     }
 
     FCA_VERBOSE(5,"Using FCA Allreduce");
-    ret = mca_coll_fca_component.fca_ops.do_all_reduce(fca_module->fca_comm, &spec);
+    ret = fca_do_all_reduce(fca_module->fca_comm, &spec);
     if (ret < 0) {
         if (ret == -EUSEMPI) {
             goto orig_allreduce;
         }
-        FCA_ERROR("Allreduce failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Allreduce failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
     return OMPI_SUCCESS;
 
 orig_allreduce:
-    return fca_module->previous_allreduce(sbuf, rbuf, count, dtype, op,
-                                       comm, fca_module->previous_allreduce_module);
+    return fca_module->previous_allreduce(sbuf, rbuf, count, dtype, op, comm,
+                                          fca_module->previous_allreduce_module);
+}
+
+/*
+ * Prepare a send buffer for allgather/allgatherv, handle packing and MPI_IN_PLACE.
+ */
+static size_t __setup_gather_sendbuf(void *sbuf, int scount,
+                                     struct ompi_datatype_t *sdtype,
+                                     struct mca_coll_fca_convertor *sconv,
+                                     void **real_sendbuf)
+{
+    size_t gap, ssize;
+
+    if (mca_coll_fca_array_size(sdtype, scount, &gap, &ssize)) {
+        *real_sendbuf = (char*) sbuf + gap;
+    } else {
+        FCA_VERBOSE(5, "Packing send buffer");
+        mca_coll_fca_convertor_create(sconv, sdtype, scount, sbuf,
+                                      MCA_COLL_FCA_CONV_SEND, real_sendbuf,
+                                      &ssize);
+        mca_coll_fca_convertor_process(sconv, 0);
+    }
+    return ssize;
+}
+
+static size_t __setup_gather_sendbuf_inplace(void *inplace_sbuf, int rcount,
+                                             struct ompi_datatype_t *rdtype,
+                                             struct mca_coll_fca_convertor *sconv,
+                                             void **real_sendbuf)
+{
+    size_t gap, ssize;
+
+    if (mca_coll_fca_array_size(rdtype, rcount, &gap, &ssize)) {
+        *real_sendbuf = (char*) inplace_sbuf + gap;
+    } else {
+        FCA_VERBOSE(5, "Packing send buffer");
+        mca_coll_fca_convertor_create(sconv, rdtype, rcount, inplace_sbuf,
+                                      MCA_COLL_FCA_CONV_SEND, real_sendbuf,
+                                      &ssize);
+        mca_coll_fca_convertor_process(sconv, 0);
+    }
+    return ssize;
 }
 
 /*
@@ -294,39 +384,56 @@ int mca_coll_fca_allgather(void *sbuf, int scount, struct ompi_datatype_t *sdtyp
 {
     mca_coll_fca_module_t *fca_module = (mca_coll_fca_module_t*)module;
 #if OMPI_FCA_ALLGATHER == 1
+    MCA_COLL_FCA_DECLARE_CONVERTOR(sconv);
+    MCA_COLL_FCA_DECLARE_CONVERTOR(rconv);
     fca_gather_spec_t spec = {0,};
+    size_t rgap, rsize;
+    ptrdiff_t rdtype_extent;
+    ssize_t total_rcount;
     int ret;
 
-    spec.sbuf = sbuf;
-    spec.rbuf = rbuf;
-    spec.size = mca_coll_fca_get_buf_size(sdtype, scount, scount);
-
-    if (spec.size < 0 || spec.size > fca_module->fca_comm_caps.max_payload ||
-        !FCA_DT_IS_CONTIGUOUS_MEMORY_LAYOUT(rdtype, ompi_comm_size(comm))) {
-        FCA_VERBOSE(5, "Unsupported allgather operation size %d, using fallback\n",
-                    spec.size);
-        goto orig_allgather;
+    /* Setup send buffer */
+    if(sbuf == MPI_IN_PLACE ) {
+        FCA_DT_EXTENT(rdtype, &rdtype_extent);
+        spec.size = __setup_gather_sendbuf_inplace(
+                        (char *)rbuf + rcount * fca_module->rank * rdtype_extent,
+                        rcount, rdtype, &sconv, &spec.sbuf);
+    } else {
+        spec.size = __setup_gather_sendbuf(sbuf, scount, sdtype, &sconv, &spec.sbuf);
     }
 
-    if (spec.size != mca_coll_fca_get_buf_size(rdtype, rcount, rcount)) {
-        FCA_VERBOSE(5, "Unsupported allgather: send_size != recv_size\n");
-        goto orig_allgather;
+    /* Setup recv buffer */
+    total_rcount = ompi_comm_size(comm) * rcount;
+    if (mca_coll_fca_array_size(rdtype, total_rcount, &rgap, &rsize) && rgap == 0) {
+        spec.rbuf = rbuf;
+    } else {
+        mca_coll_fca_convertor_create(&rconv, rdtype, total_rcount, rbuf,
+                                      MCA_COLL_FCA_CONV_RECV, &spec.rbuf, &rsize);
     }
 
-    if (MPI_IN_PLACE == spec.sbuf) {
-        FCA_VERBOSE(10, "Using MPI_IN_PLACE for sbuf");
-        spec.sbuf = (char*)spec.rbuf + spec.size * fca_module->rank;
-    }
 
-    FCA_VERBOSE(5,"Using FCA Allgather");
-    ret = mca_coll_fca_component.fca_ops.do_allgather(fca_module->fca_comm, &spec);
+    /* Call FCA Allgather */
+    FCA_VERBOSE(5,"Using FCA Allgather size");
+    ret = fca_do_allgather(fca_module->fca_comm, &spec);
+
+    /* Destroy convertors if operation failed */
     if (ret < 0) {
+        mca_coll_fca_convertor_destroy(&sconv);
+        mca_coll_fca_convertor_destroy(&rconv);
         if (ret == -EUSEMPI) {
             goto orig_allgather;
         }
-        FCA_ERROR("Allgather failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Allgather failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
+
+    /* Unpack data and clean up convertor */
+    mca_coll_fca_convertor_destroy(&sconv);
+    if (mca_coll_fca_convertor_valid(&rconv)) {
+        FCA_VERBOSE(5, "Unpacking Allgather receive buffer");
+        mca_coll_fca_convertor_process(&rconv, 0);
+        mca_coll_fca_convertor_destroy(&rconv);
+    }
     return OMPI_SUCCESS;
 
 orig_allgather:
@@ -335,7 +442,6 @@ orig_allgather:
                                           comm, fca_module->previous_allgather_module);
 }
 
-
 int mca_coll_fca_allgatherv(void *sbuf, int scount,
                            struct ompi_datatype_t *sdtype,
                            void *rbuf, int *rcounts, int *disps,
@@ -345,49 +451,94 @@ int mca_coll_fca_allgatherv(void *sbuf, int scount,
 {
     mca_coll_fca_module_t *fca_module = (mca_coll_fca_module_t*)module;
 #if OMPI_FCA_ALLGATHER == 1
+    MCA_COLL_FCA_DECLARE_CONVERTOR(sconv);
+    MCA_COLL_FCA_DECLARE_CONVERTOR(rconv);
     fca_gatherv_spec_t spec;
-    int relemsize;
+    size_t rgap, rsize;
+    int sum_rcounts;
+    ptrdiff_t rdtype_extent;
     int comm_size;
+    int relemsize;
+    size_t displ;
     int i, ret;
 
     comm_size = ompi_comm_size(fca_module->comm);
+    FCA_DT_EXTENT(rdtype, &rdtype_extent);
 
-    spec.sbuf = sbuf;
-    spec.rbuf = rbuf;
-    spec.sendsize = mca_coll_fca_get_buf_size(sdtype, scount, scount);
-
-    if (spec.sendsize < 0 || spec.sendsize > fca_module->fca_comm_caps.max_payload ||
-        !FCA_DT_IS_CONTIGUOUS_MEMORY_LAYOUT(rdtype, ompi_comm_size(comm))) {
-        FCA_VERBOSE(5, "Unsupported allgatherv operation size %d, using fallback\n",
-                    spec.sendsize);
-        goto orig_allgatherv;
+    /* Setup send buffer */
+    if(sbuf == MPI_IN_PLACE ) {
+        spec.sendsize = __setup_gather_sendbuf_inplace(
+                                (char *)rbuf + disps[fca_module->rank] * rdtype_extent,
+                                rcounts[fca_module->rank], rdtype, &sconv, &spec.sbuf);
+    } else {
+        spec.sendsize = __setup_gather_sendbuf(sbuf, scount, sdtype, &sconv, &spec.sbuf);
     }
 
+    /* Allocate alternative recvsizes/displs on the stack, which will be in bytes */
     spec.recvsizes = alloca(sizeof *spec.recvsizes * comm_size);
     spec.displs = alloca(sizeof *spec.displs * comm_size);
 
-    /* convert MPI counts which depend on dtype) to FCA sizes (which are in bytes) */
-    relemsize = mca_coll_fca_get_buf_size(rdtype, 1, comm_size);
+    /* Calculate the size of receive buffer */
+    sum_rcounts = 0;
     for (i = 0; i < comm_size; ++i) {
-        spec.recvsizes[i] = rcounts[i] * relemsize;
-        spec.displs[i] = disps[i] * relemsize;
+        sum_rcounts += rcounts[i];
     }
 
-    if (MPI_IN_PLACE == spec.sbuf) {
-        FCA_VERBOSE(10, "Using MPI_IN_PLACE for sbuf");
-        spec.sbuf = (char *)spec.rbuf + spec.displs[fca_module->rank];
+    /* convert MPI counts which depend on dtype) to FCA sizes (which are in bytes) */
+    if (mca_coll_fca_array_size(rdtype, sum_rcounts, &rgap, &rsize) && rgap == 0) {
+        spec.rbuf = rbuf;
+        for (i = 0; i < comm_size; ++i) {
+            spec.recvsizes[i] = rcounts[i] * rdtype_extent;
+            spec.displs[i] = disps[i] * rdtype_extent;
+        }
+    } else {
+        /*
+         * Reorder and remove gaps in displs - we want to allocate as little memory
+         * as possible, and we should unpack one-by-one anyway.
+         */
+        FCA_VERBOSE(5, "Reordering AllgatherV displacements");
+        mca_coll_fca_convertor_create(&rconv, rdtype, sum_rcounts, rbuf,
+                                      MCA_COLL_FCA_CONV_RECV, &spec.rbuf, &rsize);
+        assert(rsize % sum_rcounts == 0);
+        relemsize = rsize / sum_rcounts;
+
+        displ = 0;
+        for (i = 0; i < comm_size; ++i) {
+            spec.recvsizes[i] = rcounts[i] * relemsize;
+            spec.displs[i] = displ;
+            displ += spec.recvsizes[i];
+        }
+        assert(displ == rsize);
     }
 
+    /* Call FCA AllgatherV */
     FCA_VERBOSE(5,"Using FCA Allgatherv");
-    ret = mca_coll_fca_component.fca_ops.do_allgatherv(fca_module->fca_comm, &spec);
+    ret = fca_do_allgatherv(fca_module->fca_comm, &spec);
 
+    /* Destroy convertors if operation failed */
     if (ret < 0) {
+        mca_coll_fca_convertor_destroy(&sconv);
+        mca_coll_fca_convertor_destroy(&rconv);
         if (ret == -EUSEMPI) {
             goto orig_allgatherv;
         }
-        FCA_ERROR("Allgatherv failed: %s", mca_coll_fca_component.fca_ops.strerror(ret));
+        FCA_ERROR("Allgatherv failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
+
+    /* Unpack data and clean up convertor */
+    mca_coll_fca_convertor_destroy(&sconv);
+    if (mca_coll_fca_convertor_valid(&rconv)) {
+        FCA_VERBOSE(5, "Unpacking AllgatherV receive buffer rdtype_extent=%ld",
+                    rdtype_extent);
+        for (i = 0; i < comm_size; ++i) {
+            mca_coll_fca_convertor_set(&rconv, rdtype,
+                                       (char*)rbuf + disps[i] * rdtype_extent,
+                                       rcounts[i]);
+            mca_coll_fca_convertor_process(&rconv, spec.displs[i]);
+        }
+        mca_coll_fca_convertor_destroy(&rconv);
+    }
     return OMPI_SUCCESS;
 
 orig_allgatherv:
