@@ -90,11 +90,11 @@ typedef struct ompi_osc_sm_module_t ompi_osc_sm_module_t;
 int ompi_osc_sm_shared_query(struct ompi_win_t *win, int rank, size_t *size, int *disp_unit, void *baseptr);
 
 int ompi_osc_sm_attach(struct ompi_win_t *win, void *base, size_t len);
-int ompi_osc_sm_detach(struct ompi_win_t *win, void *base);
+int ompi_osc_sm_detach(struct ompi_win_t *win, const void *base);
 
 int ompi_osc_sm_free(struct ompi_win_t *win);
 
-int ompi_osc_sm_put(void *origin_addr,
+int ompi_osc_sm_put(const void *origin_addr,
                           int origin_count,
                           struct ompi_datatype_t *origin_dt,
                           int target,
@@ -112,7 +112,7 @@ int ompi_osc_sm_get(void *origin_addr,
                           struct ompi_datatype_t *target_dt,
                           struct ompi_win_t *win);
 
-int ompi_osc_sm_accumulate(void *origin_addr,
+int ompi_osc_sm_accumulate(const void *origin_addr,
                                  int origin_count,
                                  struct ompi_datatype_t *origin_dt,
                                  int target,
@@ -122,15 +122,15 @@ int ompi_osc_sm_accumulate(void *origin_addr,
                                  struct ompi_op_t *op,
                                  struct ompi_win_t *win);
 
-int ompi_osc_sm_compare_and_swap(void *origin_addr,
-                                       void *compare_addr,
+int ompi_osc_sm_compare_and_swap(const void *origin_addr,
+                                       const void *compare_addr,
                                        void *result_addr,
                                        struct ompi_datatype_t *dt,
                                        int target,
                                        OPAL_PTRDIFF_TYPE target_disp,
                                        struct ompi_win_t *win);
 
-int ompi_osc_sm_fetch_and_op(void *origin_addr,
+int ompi_osc_sm_fetch_and_op(const void *origin_addr,
                                    void *result_addr,
                                    struct ompi_datatype_t *dt,
                                    int target,
@@ -138,7 +138,7 @@ int ompi_osc_sm_fetch_and_op(void *origin_addr,
                                    struct ompi_op_t *op,
                                    struct ompi_win_t *win);
 
-int ompi_osc_sm_get_accumulate(void *origin_addr,
+int ompi_osc_sm_get_accumulate(const void *origin_addr,
                                      int origin_count,
                                      struct ompi_datatype_t *origin_datatype,
                                      void *result_addr,
@@ -151,7 +151,7 @@ int ompi_osc_sm_get_accumulate(void *origin_addr,
                                      struct ompi_op_t *op,
                                      struct ompi_win_t *win);
 
-int ompi_osc_sm_rput(void *origin_addr,
+int ompi_osc_sm_rput(const void *origin_addr,
                            int origin_count,
                            struct ompi_datatype_t *origin_dt,
                            int target,
@@ -171,7 +171,7 @@ int ompi_osc_sm_rget(void *origin_addr,
                            struct ompi_win_t *win,
                            struct ompi_request_t **request);
 
-int ompi_osc_sm_raccumulate(void *origin_addr,
+int ompi_osc_sm_raccumulate(const void *origin_addr,
                                   int origin_count,
                                   struct ompi_datatype_t *origin_dt,
                                   int target,
@@ -182,7 +182,7 @@ int ompi_osc_sm_raccumulate(void *origin_addr,
                                   struct ompi_win_t *win,
                                   struct ompi_request_t **request);
 
-int ompi_osc_sm_rget_accumulate(void *origin_addr,
+int ompi_osc_sm_rget_accumulate(const void *origin_addr,
                                       int origin_count,
                                       struct ompi_datatype_t *origin_datatype,
                                       void *result_addr,
