@@ -3,6 +3,10 @@
  * Copyright (c) 2012      Sandia National Laboratories.  All rights reserved.
  * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -15,11 +19,10 @@
 #include "ompi/mca/osc/osc.h"
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
-#include "ompi/request/request.h"
 #include "ompi/class/ompi_free_list.h"
+#include "opal/util/sys_limits.h"
 
 #include "osc_sm.h"
-#include "osc_sm_request.h"
 
 static int component_open(void);
 static int component_init(bool enable_progress_threads, bool enable_mpi_threads);
@@ -110,22 +113,6 @@ component_open(void)
 static int
 component_init(bool enable_progress_threads, bool enable_mpi_threads)
 {
-    int ret;
-
-    ret = ompi_free_list_init(&mca_osc_sm_component.requests,
-                              sizeof(ompi_request_t),
-                              OBJ_CLASS(ompi_request_t),
-                              0,
-                              0,
-                              8,
-                              NULL);
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ompi_free_list_init failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        return ret;
-    }
-
     return OMPI_SUCCESS;
 }
 
@@ -221,22 +208,19 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         if (NULL == module->node_states) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
     } else {
+        unsigned long total, *rbuf;
         char *data_file;
-        int *rbuf;
-        int total, i, flag;
+        int i, flag;
         size_t pagesize;
 	size_t state_size;
 
         OPAL_OUTPUT_VERBOSE((1, ompi_osc_base_framework.framework_output,
                              "allocating shared memory region of size %ld\n", (long) size));
 
-#ifdef HAVE_GETPAGESIZE
-        pagesize = getpagesize();
-#else
-        pagesize = 4096;
-#endif
+        /* get the pagesize */
+        pagesize = opal_getpagesize();
 
-        rbuf = malloc(sizeof(int) * ompi_comm_size(module->comm));
+        rbuf = malloc(sizeof(unsigned long) * ompi_comm_size(module->comm));
         if (NULL == rbuf) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
         module->noncontig = false;
@@ -250,8 +234,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
         } else {
             total = size;
         }
-        ret = module->comm->c_coll.coll_allgather(&total, 1, MPI_INT,
-                                                  rbuf, 1, MPI_INT,
+        ret = module->comm->c_coll.coll_allgather(&total, 1, MPI_UNSIGNED_LONG,
+                                                  rbuf, 1, MPI_UNSIGNED_LONG,
                                                   module->comm,
                                                   module->comm->c_coll.coll_allgather_module);
         if (OMPI_SUCCESS != ret) return ret;
@@ -341,7 +325,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 #if HAVE_PTHREAD_CONDATTR_SETPSHARED && HAVE_PTHREAD_MUTEXATTR_SETPSHARED
         pthread_mutexattr_t mattr;
         pthread_condattr_t cattr;
-        bool blocking_fence;
+        bool blocking_fence=false;
         int flag;
 
         if (OMPI_SUCCESS != ompi_info_get_bool(info, "blocking_fence",
@@ -349,7 +333,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
             goto error;
         }
 
-        if (blocking_fence) {
+        if (flag && blocking_fence) {
             ret = pthread_mutexattr_init(&mattr);
             ret = pthread_mutexattr_setpshared(&mattr, PTHREAD_PROCESS_SHARED);
             if (ret != 0) {
@@ -444,7 +428,7 @@ ompi_osc_sm_attach(struct ompi_win_t *win, void *base, size_t len)
 
 
 int
-ompi_osc_sm_detach(struct ompi_win_t *win, void *base)
+ompi_osc_sm_detach(struct ompi_win_t *win, const void *base)
 {
     ompi_osc_sm_module_t *module =
         (ompi_osc_sm_module_t*) win->w_osc_module;
