@@ -20,7 +20,7 @@
 #include "oshmem/mca/memheap/base/base.h"
 #include "orte/mca/errmgr/errmgr.h"
 
-mca_memheap_base_module_t mca_memheap;
+mca_memheap_base_module_t mca_memheap = {0};
 
 /**
  * Function for weeding out memheap components that shouldn't be executed.
