@@ -22,24 +22,26 @@ void oshmem_output_verbose(int level, int output_id, const char* prefix,
 {
     va_list args;
     char *buff, *str;
-    int ret;
+    int ret = 0;
 
-    UNREFERENCED_PARAMETER(ret);
+    if (level < opal_output_get_verbosity(output_id)) {
+        UNREFERENCED_PARAMETER(ret);
 
-    va_start(args, format);
+        va_start(args, format);
 
-    ret = vasprintf(&str, format, args);
-    assert(-1 != ret);
+        ret = vasprintf(&str, format, args);
+        assert(-1 != ret);
 
-    ret = asprintf(&buff, "%s %s", prefix, str);
-    assert(-1 != ret);
+        ret = asprintf(&buff, "%s %s", prefix, str);
+        assert(-1 != ret);
 
-    opal_output_verbose(level, output_id, buff, file, line, function);
+        opal_output(output_id, buff, file, line, function);
 
-    va_end(args);
+        va_end(args);
 
-    free(buff);
-    free(str);
+        free(buff);
+        free(str);
+    }
 }
 
 void oshmem_output(int output_id, const char* prefix, const char* file,
